// Extrude along path with scaling
// Modification of Andy Thaller's Extrude Along Path plugin
// AndyThaller's original notes follow:
// <BEGIN>
/* Extrude along path - An AC3D plugin
 *
 * Copyright (C) 2001 Andy Thaller <thaller@ph.tum.de>
 *
 * For more information, see http://www.ph.tum.de/~thaller/extrude/
 *
 * 08 apr 2002: initial release
 * 
 * All rights reserved.
 
 9th Sept 2005: updated for AC3D 5 - Inivis 
 */
// </END>

// Modifications by Dennis Hawthorne
// Modified sections of code have been marked with // DDH

#include <math.h>
#include <stdio.h>
// added string lib for linux (little_penguin)
#include <string.h>
//
#ifdef _WINDOWS
#define WINDOWS
#endif
#include "ac_plugin.h"

#define ME "extrude_along_path(): "

#ifdef WIN32
#include <float.h>
#define finite _finite
#endif


int debug= 0;

void eap_get_vector(Vertex *p1, Vertex *p2, Point3 *v)
{
  v->x= p2->x-p1->x;
  v->y= p2->y-p1->y;
  v->z= p2->z-p1->z;
}

double eap_get_vector_product(Point3 *vec1,Point3 *vec2)
{
  return vec1->x*vec2->x + vec1->y*vec2->y + vec1->z*vec2->z;
}

double eap_get_vector_length(Point3 *vec)
{
  return sqrt(vec->x*vec->x + vec->y*vec->y + vec->z*vec->z);
}

double eap_get_vector_cos_angle(Point3 *vec1,Point3 *vec2)
{
  double l1= eap_get_vector_length(vec1);
  double l2= eap_get_vector_length(vec2);
  if (l1==0.0||l2==0.0) return 0.0;
  return eap_get_vector_product(vec1,vec2)/(l1*l2);
}

double eap_get_vector_cos_angleV(Vertex *vec1,Vertex *vec2)
{
  Point3 v1, v2;
  v1.x= vec1->x; v1.y= vec1->y; v1.z= vec1->z;
  v2.x= vec2->x; v2.y= vec2->y; v2.z= vec2->z;
  return eap_get_vector_cos_angle(&v1,&v2);
}

void eap_get_vector_axis(Point3 *vec1,Point3 *vec2, Point3 *axis)
{
  double l;
  if (!axis) return;
  CROSSPRODUCT(vec1,vec2,axis);
  /*
  axis->x= vec1->y*vec2->z - vec1->z*vec2->y;
  axis->y= vec1->z*vec2->x - vec1->x*vec2->z;
  axis->z= vec1->x*vec2->y - vec1->y*vec2->x;
  */
  l= eap_get_vector_length(axis); if (l==0.0) return;
  axis->x/= l; axis->y/= l; axis->z/= l;
}

/* make a List of Vertex objects from the surface vertexes: */
void eap_vtx_list_get_centre(List *list, Point3 *c)
{
  List *p;
  int np= 0;
  c->x= c->y= c->z= 0.0;
  list_for_each(list,p) {
    Point3 *v= (Point3*)p->data;
    c->x+= v->x;
    c->y+= v->y;
    c->z+= v->z;
    np++;
  }
  c->x /= np;
  c->y /= np;
  c->z /= np;
  if (debug) 
    fprintf(stderr,"eap_vtx_list_get_centre(): (%f,%f,%f)\n",c->x,c->y,c->z);
}

/* make a List of Vertex objects from the surface vertexes: */
void eap_vtx_list_get_normal(List *list, Point3 *c, Point3 *n)
{
  List *p;
  Point3 a,b,nn;
  double l;
  a= *((Point3*)list->data);
  a.x-= c->x;
  a.y-= c->y;
  a.z-= c->z;
  n->x= n->y= n->z= 0.0;
  list_for_each(list->next,p) {
    b= *((Point3*)p->data);
    b.x-= c->x;
    b.y-= c->y;
    b.z-= c->z;
    eap_get_vector_axis(&a,&b,&nn);
    n->x+= nn.x;
    n->y+= nn.y;
    n->z+= nn.z;
    a= b;
  }
  l= eap_get_vector_length(n);
  n->x /= l;
  n->y /= l;
  n->z /= l;
  if (debug) 
    fprintf(stderr,"eap_vtx_list_get_normal(): (%f,%f,%f)\n",n->x,n->y,n->z);
}

/* make a List of Vertex objects from the surface vertexes: */
List *eap_vtx_list_from_surface(Surface *surface)
{
  List *p, *dest= NULL;
  int i,n=0;
  list_for_each(surface->vertlist,p) {
    list_add_item(&dest,SVERTEX(p->data));
    if (debug)
      fprintf(stderr,"eap_vtx_list_from_surface1(): #%03d : (%f,%f,%f)\n",n++,
	      SVERTEX(p->data)->x,SVERTEX(p->data)->y,SVERTEX(p->data)->z);
  }
  // DDH: REMOVED COMMENTED BLOCK (surface_get_vertex() no longer supported in AC3D):
  /*
  if (debug)
    for (i=0; i<n; i++) {
      Point3 *pp= (Point3 *)surface_get_vertex(surface, i);
      if (debug) 
	fprintf(stderr,"eap_vtx_list_from_surface2(): #%03d : (%f,%f,%f)\n",i,
		pp->x,pp->y,pp->z);
    }
    */
  return dest;
}

/* copy a List of Vertex objects and add vec to all vertices */
List *eap_vtx_list_move(List *orig, Point3 *vec)
{
  List *p, *dest= NULL;
  list_for_each(orig,p) {
    Vertex *v= (Vertex*)p->data;
    list_add_item(&dest,new_vertex_set(v->x+vec->x,v->y+vec->y,v->z+vec->z));
  }
  return dest;
}

/* copy a List of Vertex objects and add vec to all vertices */
// DDH: Added "double scaleby" param to scale the vertices
List *eap_vtx_list_rotate(List *orig, Point3 *center, Point3 *axis, double C, double scaleby)
{
  double S;
  double a= axis->x, b= axis->y, c= axis->z;
  double x0= center->x, y0= center->y, z0= center->z;
  List *p, *dest= NULL;
  if (debug)
    fprintf(stderr,ME"rotating with center (%f,%f,%f) with S=%e\n",
	    x0,y0,z0,C);
  if (C<0.0) C= 0.0; if (C>1.0) C= 1.0; S= sqrt(1.0-C*C);
  list_for_each(orig,p) {
    Vertex *v= (Vertex*)p->data;
    // DDH: Scale by our input BEGIN
    v->x = (( v->x - center->x ) * scaleby ) + center->x;
    v->y = (( v->y - center->y ) * scaleby ) + center->y;
    v->z = (( v->z - center->z ) * scaleby ) + center->z;
    // DDH: Scale by our input END
    double x= v->x-x0, y= v->y-y0, z= v->z-z0;
    double nx,ny,nz;
    nx= ((1-C)*a*a+ C )*x + ((1-C)*b*a-c*S)*y + ((1-C)*c*a+b*S)*z + x0;
    ny= ((1-C)*a*b+c*S)*x + ((1-C)*b*b+ C )*y + ((1-C)*c*b-a*S)*z + y0;
    nz= ((1-C)*a*c-b*S)*x + ((1-C)*b*c+a*S)*y + ((1-C)*c*c+ C )*z + z0;
    list_add_item(&dest,new_vertex_set(nx,ny,nz));
  }
  return dest;
}

// DDH: Added function to scale a template BEGIN
/* copy a List of Vertex objects and scale */
List *eap_vtx_list_scale(List *orig, Point3 *center, double scaleby)
{
  List *p, *dest= NULL;
  list_for_each(orig,p) {
    Vertex *v= (Vertex*)p->data;
    v->x = (( v->x - center->x ) * scaleby ) + center->x;
    v->y = (( v->y - center->y ) * scaleby ) + center->y;
    v->z = (( v->z - center->z ) * scaleby ) + center->z;
    list_add_item(&dest,new_vertex_set(v->x,v->y,v->z));
  }
  return dest;
}
// DDH: Added function to scale a template END

int eap_matrix_xlines(double *d, int l1, int l2)
{
  int i,w=4;
  double ftmp;

  for (i= 0; i<w; i++) {
    ftmp= d[l1*w+i];
    d[l1*w+i]= d[l2*w+i];
    d[l2*w+i]= ftmp;
  }
  return 1;
}

int eap_calc_utri(double *d)
{
  int i,j,w=4,h=3,l,k,itmp, l1, l2;
  double min, ftmp;
  int done[3] = { -1, -1, -1 };

  for (i= 0; i<w-1; i++) {
    /* find the line l with the min value in column i:) */
    min= -1; l= -1;
    for (j= 0; j<h; j++) {
      ftmp= fabs(d[i+j*w]); 
      if ((min<0 || ftmp<min) && ftmp>0 && done[j]<0) { min= ftmp; l= j; }
    }
    if (l>=0) {
      done[l]= i; /* Normalize line l: */
      ftmp= d[l*w+i]; for (k= 0; k<w; k++) d[l*w+k]/= -ftmp;
      for (j= 0; j<h; j++) /* for every line except the l-th: */
        if (j!=l) {
          /* remeber the factor to multiply the l-th line with: */
          ftmp= d[j*w+i];
          for (k= 0; k<w; k++)
            d[j*w+k]= d[j*w+k]*d[l*w+i]- ftmp* d[l*w+k];
        } 
    }
  }
  for (j= 0; j<h; j++) { /* sort the matrix lines: */
    l1= j; l2= -1;
    while (l2!=j) if (done[j]>=0) {
      l2= done[j];
      eap_matrix_xlines(d,l1,l2);
      itmp= done[l1]; done[l1]= done[l2]; done[l2]= itmp; 
    } else l2= l1;
  }
  return 1;
}

/* Intersect two lines
 *
 * line1 contains a1 and has direction b1= c1-a1
 * line2 contains a2 and has direction b2= c2-a2
 *
 * returns the point X of intersection, or the point in the center of the
 * shortest connection between line1 and line2
 */
Vertex *vtx_line_intersection(Vertex a1, Vertex c1, Vertex a2, Vertex c2)
{
  Vertex vm; 
  double cosa, d;
  double m[12], r,s,t;
  Vertex b1= c1, b2= c2, v= a2, b1b2, a1a2= a2, p1,p2;
  /* b1 and b2 are points of the respective lines, make them directions: */
  /* b1-= a1; b2-= a2 */
  SUBTRACTPOINTS(&a1,&b1); SUBTRACTPOINTS(&a2,&b2);
  SUBTRACTPOINTS(&a1,&v); SUBTRACTPOINTS(&a1,&a1a2);

  /* we cannot operate on parallel lines: */
  cosa=  eap_get_vector_cos_angle((Point3*)&b1,(Point3*)&b2);
  if (cosa==1.0) return new_vertex_set(c1.x,c1.y,c1.z);
  CROSSPRODUCT(&b1,&b2,&b1b2);
  d= fabs(DOTPRODUCT(&v,&b1b2));

  /* setup the matrix for the equation system and unify (solve) it */
  m[ 0]= b1.x; m[ 1]= -b2.x; m[ 2]= b1b2.x; m[ 3]= a1a2.x;
  m[ 4]= b1.y; m[ 5]= -b2.y; m[ 6]= b1b2.y; m[ 7]= a1a2.y;
  m[ 8]= b1.z; m[ 9]= -b2.z; m[10]= b1b2.z; m[11]= a1a2.z;
  eap_calc_utri(m); r= -m[3]; s= -m[7]; t= -m[11];
  
  /* calculate the two points which are nearest to each other: */
  p1= b1; MULTIPLYPOINT(&p1,r); ADDPOINTS(&a1,&p1);
  p2= b2; MULTIPLYPOINT(&p2,s); ADDPOINTS(&a2,&p2);

  /* return the midpoint of those two: */
  MIDPOINT(&p1,&p1,&vm); return new_vertex_set(vm.x,vm.y,vm.z);
}

List *eap_eap_vtx_list_get_intersection(List *p1, List *p2, List *p3, List *p4)
{
  List *dest= NULL;
  for( ;p1&&p2&&p3&&p4; p1=p1->next, p2=p2->next, p3=p3->next, p4=p4->next) {
    Vertex *v1= (Vertex*)p1->data; Vertex *v2= (Vertex*)p2->data;
    Vertex *v3= (Vertex*)p3->data; Vertex *v4= (Vertex*)p4->data;
    list_add_item(&dest,vtx_line_intersection(*v1,*v2,*v3,*v4));
  }
  return dest;
}

void eap_vtx_list_add_to_object(ACObject *ob, List *orig)
{
  List *p;
  list_for_each(orig,p) {
    object_add_new_vertex(ob,(Point3*)p->data);
  }
}
#define COSADEG(A) (acos(A)*180/M_PI)
// DDH: Changed call to include argc/argv
static void eap_extrude_along_path( int argc, const char* argv[] )
{

  // DDH: Parse args BEGIN
  if( argc < 2 )
  {
    message_dialog( "Insufficient parameters passed to extrude/scale plugin" );
    return;
  }
  float f = 0;
  if( sscanf( argv[1], "%g", &f ) != 1 )
  {
    message_dialog( "Error parsing scale amount (%s)", argv[1] );
    return;
  }
  double total_scale_amt = (( double )f ) * 0.01;
  // DDH: Parse args END

  List *vl, *ol, *p;
  ACObject *tmpl, *path;
  Surface *tmplsurf= 0, *pathsurf= 0; 
  
  int mode, closed_path, closed_tmpl, tnv,u,v, tnu=0;

  List *vl0, *vl1, *vl2, *vl3, *vl4; /* List of Point3 */
  List *pathvertlist;                /* List of Vertex */
  Vertex *P1, *P2, *P3;
  Point3 v1, v2, axis, PP, norm;
  double cosa;
  ACObject *ob= new_object(OBJECT_NORMAL);

  // DDH: Changed to ac_selection_get_objects_ordered() from ac_selection_get_objects()
  ol = ac_selection_get_objects_ordered();
  
  if (debug) 
    fprintf(stderr,ME"selection contains %d objects\n",
	   list_count(ol));

  /* make sure we can actually use the selected objects: */

  if (list_count(ol)!=2) {
    fprintf(stderr,ME"can only operate on exactly two objects!\n");
    return;
  }
  list_for_each(ol,p) {
    int numvert, numsurf, numchildren;
    List *points, *surfaces, *children;
    ac_object_get_contents((ACObject*)p->data, &numvert, &numsurf, 
			   &numchildren, &points, &surfaces, &children);
    eap_vtx_list_from_surface((Surface*)(surfaces->data));
    fprintf(stderr,"\n");
    if (numsurf!=1 || numchildren!=0) {
      fprintf(stderr,ME"must have exactly one surface and no children!\n");
      return;
    }
    if (ac_object_get_type((ACObject*)p->data)!=OBJECT_NORMAL) {
      fprintf(stderr,ME"object must be of type OBJECT_NORMAL!\n");
      return;
    }
  }

  tmpl= (ACObject*) (ol->data);
  path= (ACObject*) (ol->next->data);
  object_calc_normals_force(tmpl);
  tmplsurf= (Surface*)(ac_object_get_surfacelist(tmpl)->data);
  pathsurf= (Surface*)(ac_object_get_surfacelist(path)->data);
  closed_tmpl= (surface_get_type(tmplsurf)!=SURFACE_LINE);
  closed_path= (surface_get_type(pathsurf)!=SURFACE_LINE);
  
  if (debug) {
    fprintf(stderr,ME"tmpl is %s\n",(closed_tmpl)?"closed":"open");
    fprintf(stderr,ME"path is %s\n",(closed_path)?"closed":"open");
  }

  if (pathsurf->numvert<3) {
    fprintf(stderr,ME"path must have at least 3 vertices!\n");
    return;
  }
  pathvertlist= list_clone(pathsurf->vertlist);
  P1= ((SVertex*)list_get_item(pathvertlist,0))->v;
  P2= ((SVertex*)list_get_item(pathvertlist,1))->v;
  P3= ((SVertex*)list_get_item(pathvertlist,2))->v;

  /* the origin for the new object: */
  PP.x= 0; PP.y= 0; PP.z= 0;
  eap_get_vector(P1,P2,&v1); 
  eap_get_vector(P2,P3,&v2); 

  /* first, get the list of vertices in the template and align it with
   * the first segment of the path 
   */
  vl0= eap_vtx_list_from_surface(tmplsurf);
  eap_vtx_list_get_centre(vl0,&PP);
  eap_vtx_list_get_normal(vl0,&PP,&norm);
  if (DOTPRODUCT(&norm,&v1)<0.0) NEGATEPOINT(&norm);
  cosa= eap_get_vector_cos_angle(&norm,&v1);

  if (cosa==1.0 || !finite(cosa)) {
    vl1= list_clone(vl0);
    if (debug)
      fprintf(stderr,ME"template and first segment of path are aligned.\n");
  } else
  {
    eap_get_vector_axis(&norm,&v1,&axis);
    // DDH: Added final scaleby param to eap_vtx_list_rotate()
    vl1= eap_vtx_list_rotate(vl0,&PP,&axis,cosa,1.0);
    if (debug)
      fprintf(stderr,ME"aligning template with first segment of path "
	      "axis  (%f,%f,%f) angle %f.\n",
	      axis.x,axis.y,axis.z,COSADEG(cosa));
  }
  if (!closed_path) {
    /* The path object is not closed, so add the first vertex list. */
    if (debug) fprintf(stderr,ME"store the tmpl.\n");
    eap_vtx_list_add_to_object(ob,vl1); tnu++;
  } else {
    /* It's closed, expand the path with the first two segments to make
     * things _much_ easier... */
    list_add_item(&pathvertlist,list_get_item(pathvertlist,0));
    list_add_item(&pathvertlist,list_get_item(pathvertlist,1));
  }

  /* now move the template along the path */
  vl2= eap_vtx_list_move(vl1,&v1);

  // DDH: Create an array of scaling amounts for each point in the original path BEGIN
  int num_path_pts = list_count( pathvertlist );
  double *path_scaling = new double[num_path_pts];
  double *path_distance = new double[num_path_pts];
  path_scaling[0] = 1.0;
  path_distance[0] = 0.0;
  // Calculate distance from first to last point
  double path_length_total = 0;
  int ctr = 1;
  for( p = pathvertlist; p->next != NULL; p = p->next )
  {
     Vertex *v1 = ((SVertex*)(p->data))->v;
     Vertex *v2 = ((SVertex*)(p->next->data))->v;
     double dx = v1->x - v2->x;
     double dy = v1->y - v2->y;
     double dz = v1->z - v2->z;
     path_distance[ctr] = path_distance[ctr - 1] + sqrt( dx * dx + dy * dy + dz * dz );
     ++ctr;
  }
  path_length_total = path_distance[ctr - 1];
  if( path_length_total == 0 )
  {
     path_length_total = 0.0001;
  }
  // Calculate progress to each point
  ctr = 1;
  for( p = pathvertlist->next; p != NULL; p = p->next )
  {
     // TODO: Needs to be scaled down based on distance, not number of points
     double pct_to_end = (( double )ctr ) / (( double )( num_path_pts - 1 ));
     //double point_scale = 1.0 - (( 1.0 -  total_scale_amt ) * pct_to_end );
     double point_scale = 1.0 - (( 1.0 -  total_scale_amt ) * ( path_distance[ctr] / path_length_total ));
     path_scaling[ctr] = point_scale;
     ++ctr;
  }
  delete[] path_distance;
  // Adjust scaling so it is additive rather than absolute
  for( int ix = num_path_pts - 1; ix > 0; --ix )
  {
     path_scaling[ix] = path_scaling[ix] / path_scaling[ix - 1];
  }
  ctr = 1;
  // DDH: Create an array of scaling amounts for each point in the original path END

  for (p = pathvertlist->next->next; p != NULL; p = p->next) {
    /* to create vl2, determine angle and axis of rotation to transform a path
     * segment (v1) to the next one (v2), and apply this transformation to the
     * template */
    PP.x+= v1.x; PP.y+= v1.y; PP.z+= v1.z;
    if (debug) fprintf(stderr,ME"PP is (%f,%f,%f).\n",PP.x,PP.y,PP.z);
    cosa= eap_get_vector_cos_angle(&v1,&v2);

    if (cosa==1.0 || !finite(cosa)) {
      // no axis could be determined here, so we treat it as collinear
      if (debug) fprintf(stderr,ME"store vl3 and advance.\n");
      vl3= eap_vtx_list_scale(vl2,&PP,path_scaling[ctr]);
      vl4= eap_vtx_list_move(vl3,&v2);
      eap_vtx_list_add_to_object(ob,vl3); tnu++;
    } else
    {
      List *store;
      eap_get_vector_axis(&v1,&v2,&axis);
      if (debug) fprintf(stderr,ME"rotate around (%f,%f,%f).\n",
			 axis.x,axis.y,axis.z);
      // DDH: Added final scaleby param to eap_vtx_list_rotate() BEGIN
      vl3= eap_vtx_list_rotate(vl2,&PP,&axis,cosa,path_scaling[ctr]);
      // DDH: Added final scaleby param to eap_vtx_list_rotate() END
      vl4= eap_vtx_list_move(vl3,&v2);
      if (debug) fprintf(stderr,ME"intersect, store and advance.\n");
      store= eap_eap_vtx_list_get_intersection(vl1,vl2,vl3,vl4);
#if 0
      eap_vtx_list_add_to_object(ob,vl2); tnu++;
      eap_vtx_list_add_to_object(ob,vl3); tnu++;
#else
      eap_vtx_list_add_to_object(ob,store); tnu++;
#endif
      // DDH: Increment scale index counter
      ++ctr;
    }
    /* while we still have path segments, copy all relevant stuff and get the
     * next segment's stuff: */
    P1= P2; P2= P3; P3= 0; v1= v2;
    list_free(&vl1); list_free(&vl2);
    vl1= vl3; vl2= vl4;
    if (p->next) {
      P3= ((SVertex*)(p->next->data))->v;
      eap_get_vector(P2,P3,&v2);
    }
  }
  if (closed_path) {
    // do nothing for the moment
  } else {
    // DDH: Scale final point
    vl3= eap_vtx_list_scale(vl2,&PP,path_scaling[ctr]);
    eap_vtx_list_add_to_object(ob,vl2); tnu++;
  }

  // DDH: Done with path_scaling
  delete[] path_scaling;

#define MY_SURFACE_ADD_VERTICE(u,v) surface_add_vertex(s,ac_object_get_vertex_at_index(ob,(u)*tnv+(v)),0,0)

  tnv= tmplsurf->numvert;
  for (u= 0; u<tnu-1; u++) {
    for (v= 0; v<tnv-1; v++) {
      Surface *s = new_surface();
      MY_SURFACE_ADD_VERTICE(u,v);
      MY_SURFACE_ADD_VERTICE(u,v+1);
      MY_SURFACE_ADD_VERTICE(u+1,v+1);
      MY_SURFACE_ADD_VERTICE(u+1,v);
      object_add_surface(ob, s);
    }
    if (closed_tmpl) {
      Surface *s = new_surface();
      MY_SURFACE_ADD_VERTICE(u,tnv-1);
      MY_SURFACE_ADD_VERTICE(u,0);
      MY_SURFACE_ADD_VERTICE(u+1,0);
      MY_SURFACE_ADD_VERTICE(u+1,tnv-1);
      object_add_surface(ob, s);
    }    
  }
#if 0 /* this will lead to crap in most cases - connect by hand, instead */
  if (closed_path) {
    for (v= 0; v<tnv-1; v++) {
      Surface *s = new_surface();
      MY_SURFACE_ADD_VERTICE(tnu-1,v);
      MY_SURFACE_ADD_VERTICE(tnu-1,v+1);
      MY_SURFACE_ADD_VERTICE(0,v+1);
      MY_SURFACE_ADD_VERTICE(0,v);
      object_add_surface(ob, s);
    }
    if (closed_tmpl) {
      Surface *s = new_surface();
      MY_SURFACE_ADD_VERTICE(tnu-1,tnv-1);
      MY_SURFACE_ADD_VERTICE(tnu-1,0);
      MY_SURFACE_ADD_VERTICE(0,0);
      MY_SURFACE_ADD_VERTICE(0,tnv-1);
      object_add_surface(ob, s);
    }    
  }
#endif
#undef MY_SURFACE_ADD_VERTICE

  object_calc_normals_force(ob);
  add_undoable_world_add_object("Extrude along path", ob);

redraw_all();
//  object_add_child(ac_get_world(), ob);
//  object_calc_normals_force(ob);
//  clear_selection();
//  ac_select_object(ob);
}


// DDH: Changed init BEGIN
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{
  char *path = ( char * )strdup( d->plugin_dir_path );
#ifdef WINDOWS
  ac_unbackslash_string( path );
#endif
  tcl_command( "source \"%s/extpath_scale.tcl\"", path );
  ac_add_command_full("eap_extrude_along_path_scale", ( void * )eap_extrude_along_path, 0, "argv", "<scale_amount>", "Extrude path with scaling" );
  ac_add_tools_menu_item("Extrude along path with scaling...", 
			 "ddhextpathscale_dialog",
			 "Extrude a surface along a path (line) with scaling across the length of the path");
  return(0);
}
// DDH: Changed init END


AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
  return(0);
}


AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
  return("EAP - extrude along path v1.1 by Andy Thaller/Inivis");
}

AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
  return("Adds a new menu entry 'Tools->Extrude along path'.  Extrudes a profile along a line.  Create a new object consisting of a single polyline or line serving as the template object and a path object also with only one polyline or line.\
Clear your selections and then select first the template object and afterwards add the path object to the selection by holding down the SHIFT key. Now choose 'Tools->Extrude along path' from the menu to activate the function.");
}
